/*******************************************************************************
 * Copyright 2012 University of Southern California
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * 	http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * This code was developed by the Information Integration Group as part 
 * of the Karma project at the Information Sciences Institute of the 
 * University of Southern California.  For more information, publications, 
 * and related projects, please see: http://www.isi.edu/integration
 ******************************************************************************/
package edu.isi.karma.webserver;

import java.util.HashMap;
import java.util.Map;

/**
 * Singleton class to map Workspace Ids to ExecutionController objects
 * 
 * @author szekely
 * 
 */
public class WorkspaceRegistry {

	private static WorkspaceRegistry singleton = new WorkspaceRegistry();

	private final Map<String, ExecutionController> workspaceId2ExecutionController = new HashMap<String, ExecutionController>();

	public static WorkspaceRegistry getInstance() {
		return singleton;
	}

	public void register(ExecutionController executionController) {
		workspaceId2ExecutionController.put(executionController
				.getWorkspace().getId(), executionController);
	}

	public ExecutionController getExecutionController(String workspaceId) {
		return workspaceId2ExecutionController.get(workspaceId);
	}
	
	public void deregister(String workspaceId) {
		workspaceId2ExecutionController.remove(workspaceId);
	}
}
